local addonName, addonTable = ...

addonTable.name = GetAddOnMetadata(addonName, "Title")
addonTable.version = GetAddOnMetadata(addonName, "X-Packaged-Version")

SLASH_SSB1 = '/ssb'
SSB_DB = {}
SSB_CharDB = {}

function SSB_Backdrop_OnUpdate(self)
	SSB_Backdrop:SetHeight(45)
	SSB_Backdrop:SetWidth(500)

	local hasRep = SSB_SetReputationText()
	local hasXp = SSB_SetExpText()
	local hasAp = SSB_SetAzeriteText()

	SSB_SetOffsets(hasRep, hasXp, hasAp)
end

function SSB_Backdrop_OnMouseDown(self)
	self:SetScript("OnUpdate", nil);
end

function SSB_Backdrop_OnMouseUp(self)
	self:SetScript("OnUpdate", SSB_Backdrop_OnUpdate);
end

function SSB_Backdrop_OnDragStart(self)
	if SSB_CharDB.locked then
		return
	end

	BackdropTexture:SetColorTexture(1.0, 1.0, 1.0, 0.25)
	self:StartMoving();
end

function SSB_Backdrop_OnDragStop(self)
	if SSB_CharDB.locked then
		return
	end

	BackdropTexture:SetColorTexture(0.0, 0.0, 0.0, 0.0)
	SSB_Backdrop_OnMouseUp(self); -- OnMouseUp won't fire if OnDragStart fired after OnMouseDown
	self:StopMovingOrSizing();
	
	local a, _, _, x, y = SSB_Backdrop:GetPoint()
	x = math.floor(x + 0.5)
	y = math.floor(y + 0.5)
	SSB_Backdrop:SetPoint(a, x, y)
	printf("|cff00ffffSSB|r: Dragged to %s, %s [%s]", x, y, a)
end

function SSB_Backdrop_OnEvent(self, event, arg)
	if event == "PLAYER_LOGIN" then
		SSB_Initialize()
	elseif event == "PLAYER_LOGOUT" then
		SimpleStatusBars_CharacterDB.position = SSB_GetPosition()
	end
end

function SSB_Backdrop_OnLoad(self)
	self:RegisterEvent("PLAYER_LOGIN")
	self:RegisterEvent("PLAYER_LOGOUT")
	self:RegisterForDrag("LeftButton")
	
	SlashCmdList["SSB"] = SSB_SlashCommandHandler
end

function SSB_SlashCommandHandler(msg)
	local cmd, arg1, arg2, arg3 = splitArgs(msg)

	if cmd ~= nil then
		cmd = strlower(cmd)
	end

	if cmd == "on" or cmd == "show" then
		SSB_Backdrop:Show();
	elseif cmd == "off" or cmd == "hide" then
		SSB_Backdrop:Hide();
	elseif cmd == "p" or cmd == "pos" then
		SSB_Position()
	elseif cmd == "r" or cmd == "reset" then
		SSB_Reset();
	elseif cmd == "m" or cmd == "move" then
		SSB_Move(arg1, arg2, arg3, true)
	elseif cmd == "lock" then
		SSB_Lock()
	elseif cmd == "unlock" then
		SSB_Unlock()
	elseif cmd == "profiles" then
		printm("Profiles: [ %s ]", SSB_GetProfiles())
	elseif cmd == "load" then
		SSB_LoadProfile(arg1)
	elseif cmd == "save" then
		SSB_SaveProfile(arg1)
	elseif cmd == "remove" then
		SSB_RemoveProfile(arg1)
	else
		SSB_CmdList()
	end
end

function SSB_Initialize()
	SimpleStatusBars_DB = SimpleStatusBars_DB or {}
	SimpleStatusBars_CharacterDB = SimpleStatusBars_CharacterDB or {}

	SSB_DB = SimpleStatusBars_DB
	SSB_DB.profiles = SSB_DB.profiles or {}

	SSB_CharDB = SimpleStatusBars_CharacterDB
	SSB_CharDB.position = SSB_CharDB.position or "0 0 CENTER"
	SSB_CharDB.locked = SSB_CharDB.locked or false

	local arg1, arg2, arg3 = splitArgs(SSB_CharDB.position)
	SSB_Move(arg1, arg2, arg3, false)

	printf("|cff0099ff%s |cff00ffff{SSB}|r v%s - Loaded", addonTable.name, addonTable.version)
end

function SSB_Lock()
	SSB_CharDB.locked = true
	printm("Locked")
end

function SSB_Unlock()
	SSB_CharDB.locked = false
	printm("Unlocked")
end

function SSB_LoadProfile(profileName)
	if profileName == nil or profileName == "" then
		return
	end

	local profile = SSB_DB.profiles[profileName]

	if profile == nil then
		printm("Unable to find profile |cffff9900%s", profileName)
		return
	end
	
	local arg1, arg2, arg3 = splitArgs(profile.position)

	SSB_CharDB.locked = profile.locked
	SSB_Move(arg1, arg2, arg3, false)

	local a, _, _, x, y = SSB_Backdrop:GetPoint()
	printm("Loaded profile |cffff9900%s", profileName)
end

function SSB_SaveProfile(profileName)
	if profileName == nil or profileName == "" then
		return
	end

	local profile = {}
	profile.locked = SSB_CharDB.locked
	profile.position = SSB_CharDB.position

	SSB_DB.profiles[profileName] = profile

	printm("Saved profile |cffff9900%s", profileName)
end

function SSB_RemoveProfile(profileName)
	if profileName == nil or profileName == "" then
		return
	end
	
	SSB_DB.profiles[profileName] = nil

	printm("Removed profile |cffff9900%s", profileName)
end

function SSB_SetOffsets(hasRep, hasXP, hasAP)
	local repOff = 0
	local apOff = 0

	if hasAP == false then
		repOff = repOff - 15
	end

	if hasXP == false then
		repOff = repOff - 15
		apOff = apOff - 30
	end

	if hasRep == true and repOff ~= 0 then
		ReputationParent:SetPoint("TOPLEFT", 0, repOff)
	end

	if hasAP == true and apOff ~= 0 then
		APParent:SetPoint("TOPLEFT", 0, apOff)
	end
end

function SSB_SetReputationText()
    local name, standing, barMin, barMax, barValue, factionId = GetWatchedFactionInfo()
	local paraValue, paraMax, paraQuestId, paraRewardPending = C_Reputation.GetFactionParagonInfo(factionId)

    if name == nil then
    	ReputationParent:Hide()
    	return false
    end
    	
    ReputationParent:Show()

    local standingName, r, g, b
    if standing == 1 then standingName = "Hated"; end
    if standing == 2 then standingName = "Hostile" end
    if standing == 3 then standingName = "Unfriendly" end
    if standing == 4 then standingName = "Neutral" end
    if standing == 5 then standingName = "Friendly" end
    if standing == 6 then standingName = "Honored" end
    if standing == 7 then standingName = "Revered" end
    if standing == 8 then standingName = "Exalted" end

    if C_Reputation.IsFactionParagon(factionId) then
    	local paraRank = floor(paraValue / paraMax)
    	if paraRank > 0 then
    		standingName = "Paragon Rank " .. paraRank
    	else
    		standingName = "Paragon"
    	end
    	barValue = paraValue % paraMax
    	barMin = 0
    	barMax = paraMax
    end

	local val = barValue - barMin
	local max = barMax - barMin
	local percent = 0
	local exalted = false
	
	if val == 0 and max == 0 then
		percent = 1
		exalted = true
	else
		percent = val / max
	end

	local width = ReputationBar:GetParent():GetWidth()

	ReputationBarName:SetFormattedText("|cFFFFFFFF%s", name)

	if exalted then
		ReputationName:SetFormattedText("|cFFFFFFFF%s", standingName)
	else
		ReputationName:SetFormattedText("|cFFFFFFFF%s / %s (%s)", val, max, standingName)
	end
	ReputationPercent:SetFormattedText("|cFFFFFFFF%.1f%%", percent * 100)
	ReputationBar:SetWidth(width * percent)

	if standing < 4 then ReputationBar:SetVertexColor(0.75, 0, 0) end
	if standing == 4 then ReputationBar:SetVertexColor(0.75, 0.75, 0) end
	if standing > 4 then ReputationBar:SetVertexColor(0, 0.75, 0) end

	if val == 0 and max ~= 0 then
		ReputationBar:Hide()
	else
		ReputationBar:Show()
	end

	return true
end

function SSB_SetExpText()
	local val = UnitXP("player")
	local max = UnitXPMax("player")
	local lvl = UnitLevel("player")
	local maxlvl = GetMaxPlayerLevel()
	local retVal = GetXPExhaustion()
	local percent = val/max

	if lvl == maxlvl then
		ExpParent:Hide()
		SSB_Backdrop:SetHeight(30)
		return false
	end

	ExpParent:Show()

	local width = ExpBar:GetParent():GetWidth()
	local rem = max - val;

	ExpName:SetFormattedText("|cFFFFFFFF%s / %s [%s]", val, max, rem)
	ExpPercent:SetFormattedText("|cFFFFFFFF%.1f%%", percent * 100)
	ExpBar:SetWidth(width * percent)

	if retVal == nil then
		ExpBar:SetVertexColor(0, 0.75, 0)
	else
		ExpBar:SetVertexColor(0, 0.26, 0.59)
	end

	if val == 0 then
		ExpBar:Hide()
	else
		ExpBar:Show()
	end

	return true
end

function SSB_SetAzeriteText()
	local hasActiveAzeriteItem = C_AzeriteItem.HasActiveAzeriteItem()

    if not hasActiveAzeriteItem then
		APParent:Hide();
		return false
	end

	local azeriteItem = C_AzeriteItem.FindActiveAzeriteItem()

	local xp, totalXp = C_AzeriteItem.GetAzeriteItemXPInfo(azeriteItem)
	local powerLevel = C_AzeriteItem.GetPowerLevel(azeriteItem)

	local percent = xp / totalXp
	
	local val = SSB_GetAbbr(xp, 2)
	local max = SSB_GetAbbr(totalXp, 2)
	local width = ExpBar:GetParent():GetWidth()

	APBarName:SetFormattedText("|cFFFFFFFF%s", "Heart of Azeroth")
	APName:SetFormattedText("|cFFFFFFFF%s / %s (Level %s)", val, max, powerLevel)
	APPercent:SetFormattedText("|cFFFFFFFF%.1f%%", percent * 100)
	APBar:SetWidth(width * percent)
	APBar:SetVertexColor(0.61, 0.54, 0.39)

	if val == 0 then
		APBar:Hide()
	else
		APBar:Show()
	end

	return true
end

function SSB_GetAbbr(amount, decimals)
	local f = "%." .. decimals .. "f"
	local ak = amount / 1000
	local am = ak / 1000
	local ab = am / 1000
	local sym = ""

	if ab > 1 then
		return string.format(f .. " B", ab)
	elseif am > 1 then
		return string.format(f .. " M", am)
	elseif ak > 1 then
		return string.format(f .. " K", ak)
	else
		return string.format("%d", amount)
	end
end

function SSB_CmdList()
	local a, _, _, x, y = SSB_Backdrop:GetPoint()
	printf("|cff0099ffSimpleStatusBars |cff00ffff{SSB}|r v%s", addonTable.version)
	printf("|cff00ffffProfiles|r: [ %s ]", SSB_GetProfiles())
	printf("|cff00ffffSaved Position|r: %s, %s [%s]", x, y, a)
	printf("|ccf00ffffLocked|r: %s", tostring(SSB_CharDB.locked))
	printf("|cffff9900Options |cff00ffff|r /ssb {opt}\n")
	printf("|cffffcc00   show, on|r Show panel")
	printf("|cffffcc00   hide, off|r Hide panel")
	printf("|cffffcc00   pos, p|r Show current panel position")
	printf("|cffffcc00   reset, r|r Reset panel to center")
	printf("|cffffcc00   move, m |cffff9900{x, y, anchor}|r Move panel to x, y using the anchor")
	printf("|cffff9900      anchor|r one of (case-insensitive, abbr. in |cffff9900orange|r)")
	printf("|cffffffff      |cffff9900C|rENTER |cffffcc00{default}|r, |cffff9900T|rOP, |cffff9900B|rOTTOM, |cffff9900L|rEFT, |cffff9900R|rIGHT")
	printf("|cffffffff      |cffff9900T|rOP|cffff9900R|rIGHT, |cffff9900T|rOP|cffff9900L|rEFT, |cffff9900B|rOTTOM|cffff9900R|rIGHT, |cffff9900B|rOTTOM|cffff9900L|rEFT")
	printf("|cffffcc00   lock|r Lock dragging for the panel")
	printf("|cffffcc00   unlock|r Unlock dragging for the the panel")
	printf("|cffffcc00   profiles|r List saved profiles")
	printf("|cffffcc00   save |cffff9900{profile}|r Save profile")
	printf("|cffffcc00   load |cffff9900{profile}|r Load profile")
	printf("|cffffcc00   remove |cffff9900{profile}|r Remove profile")
end

function SSB_Position()
	local a, _, _, x, y = SSB_Backdrop:GetPoint()
	printm("Positioned at %s, %s [%s]", x, y, a)
end

function SSB_Reset()
	local width = SSB_Backdrop:GetWidth()
	printm("Resetting to center")
	SSB_Backdrop:ClearAllPoints()
	SSB_Backdrop:SetPoint("CENTER", 0, 0)
end

function SSB_Move(arg1, arg2, arg3, report)
	local x = tonumber(arg1)
	local y = tonumber(arg2)
	local a = SSB_GetValidPoint(arg3)

	if x == nil or y == nil then
		return
	end

	if report then
		printm("Moving to %s, %s [%s]", x, y, a)
	end

	SSB_Backdrop:ClearAllPoints()
	SSB_Backdrop:SetPoint(a, x, y)
end

function SSB_GetValidPoint(anchor)
	local a = strupper(anchor)

	if a == "TOP" or a == "T" then return "TOP" end
	if a == "BOTTOM" or a == "B" then return "BOTTOM" end
	if a == "RIGHT" or a == "R" then return "RIGHT" end
	if a == "LEFT" or a == "L" then return "LEFT" end
	
	if a == "TOPRIGHT" or a == "TR" then return "TOPRIGHT" end
	if a == "TOPLEFT" or a == "TL" then return "TOPLEFT" end
	if a == "BOTTOMRIGHT" or a == "BR" then return "BOTTOMRIGHT" end
	if a == "BOTTOMLEFT" or a == "BL" then return "BOTTOMLEFT" end
	
	return "CENTER"
end

function SSB_GetPosition()
	local a, _, _, x, y = SSB_Backdrop:GetPoint()
	x = math.floor(x + 0.5)
	y = math.floor(y + 0.5)
	return format("%s %s %s", x, y, a)
end

function SSB_GetProfiles()
	local profiles = nil
	for key, value in pairs(SSB_DB.profiles) do
		local profile = "|cffff9900" .. key .. "|r"
		if profiles == nil then
			profiles = profile
		else
			profiles = profiles .. ", " .. profile
		end
	end

	if profiles == nil then
		return ""
	else
		return profiles
	end
end

function splitArgs(args)
	if args == nil then
		return {}
	end

	local list = {}
	local i = 0
	for arg in args:gmatch("%S+") do
		table.insert(list, arg)
	end
	return unpack(list)
end

function printm(msg, ...)
	printf("|cff00ffffSSB|r: " .. msg, ...)
end

function printf(f, ...)
	print(string.format(f, ...))
end
